@@ -36,6 +36,7 @@ except ImportError:
         kerberos = None
 
 from .utils.cli import sasctl_command
+from .utils.misc import versionadded
 from . import exceptions
 
 logger = logging.getLogger(__name__)
@@ -80,13 +81,15 @@ def _filter_password(r):
 def _filter_token(r):
     # Redact "Bearer <token>" in Authorization headers
     if hasattr(r, 'headers') and 'Authorization' in r.headers:
-        r.headers['Authorization'] = _redact(r'(?<=Bearer ).*', '[redacted]',
-                                             r.headers['Authorization'])
+        r.headers['Authorization'] = _redact(
+            r'(?<=Bearer ).*', '[redacted]', r.headers['Authorization']
+        )
 
     # Redact "Basic <base64 encoded pw> in Authorization headers.  This covers client ids & client secrets.
     if hasattr(r, 'headers') and 'Authorization' in r.headers:
-        r.headers['Authorization'] = _redact(r'(?<=Basic ).*', '[redacted]',
-                                             r.headers['Authorization'])
+        r.headers['Authorization'] = _redact(
+            r'(?<=Basic ).*', '[redacted]', r.headers['Authorization']
+        )
 
     # Redact Consul token from headers.  Should only appear when registering a new client
     if hasattr(r, 'headers') and 'X-Consul-Token' in r.headers:
@@ -94,16 +97,12 @@ def _filter_token(r):
 
     # Redact "access_token":"<token>" in response from Logon service
     if hasattr(r, '_content'):
-        r._content = _redact('(?<=access_token":")[^"]*', '[redacted]',
-                             r._content)
+        r._content = _redact('(?<=access_token":")[^"]*', '[redacted]', r._content)
 
     return r
 
 
-DEFAULT_FILTERS = [
-    _filter_password,
-    _filter_token
-]
+DEFAULT_FILTERS = [_filter_password, _filter_token]
 
 
 def current_session(*args, **kwargs):
@@ -184,14 +183,18 @@ class RestObj(dict):
 
             return result
 
-        raise AttributeError("'%s' object has no attribute '%s'" % (
-            self.__class__.__name__, item))
+        raise AttributeError(
+            "'%s' object has no attribute '%s'" % (self.__class__.__name__, item)
+        )
 
     def __repr__(self):
         headers = getattr(self, '_headers', {})
 
         return "%s(headers=%r, data=%s)" % (
-            self.__class__, headers, super(RestObj, self).__repr__())
+            self.__class__,
+            headers,
+            super(RestObj, self).__repr__(),
+        )
 
     def __str__(self):
         if 'name' in self:
@@ -244,17 +247,26 @@ class Session(requests.Session):
 
     Attributes
     ----------
-    message_log
-    filters
+    message_log : logging.Logger
+        A log to which all REST request and response messages will be sent.  Attach a handler using
+        `add_logger()` to capture these messages.
+
+    filters : list of callable
+        A collection of functions that will be called with each request and response object *prior* to logging the
+        messages, allowing any sensitive information to be removed first.
 
     """
-    def __init__(self, hostname,
-                 username=None,
-                 password=None,
-                 authinfo=None,
-                 protocol=None,
-                 port=None,
-                 verify_ssl=None):
+
+    def __init__(
+        self,
+        hostname,
+        username=None,
+        password=None,
+        authinfo=None,
+        protocol=None,
+        port=None,
+        verify_ssl=None,
+    ):
         super(Session, self).__init__()
 
         # Determine whether or not server SSL certificates should be verified.
@@ -294,6 +306,7 @@ class Session(requests.Session):
             else:
                 # Every request will generate an InsecureRequestWarning
                 from urllib3.exceptions import InsecureRequestWarning
+
                 warnings.simplefilter('default', InsecureRequestWarning)
 
         self.filters = DEFAULT_FILTERS
@@ -303,8 +316,9 @@ class Session(requests.Session):
 
         # Reuse an existing CAS connection if possible
         if swat and isinstance(hostname, swat.CAS):
-            if isinstance(hostname._sw_connection,
-                          swat.cas.rest.connection.REST_CASConnection):
+            if isinstance(
+                hostname._sw_connection, swat.cas.rest.connection.REST_CASConnection
+            ):
                 import base64
 
                 # Use the httpAddress action to retieve information about
@@ -319,12 +333,14 @@ class Session(requests.Session):
                 protocol = address.protocol
                 port = address.port
                 auth = hostname._sw_connection._auth.decode('utf-8').replace(
-                    'Basic ', '')
-                username, password = base64.b64decode(auth).decode(
-                    'utf-8').split(':')
+                    'Basic ', ''
+                )
+                username, password = base64.b64decode(auth).decode('utf-8').split(':')
             else:
-                raise ValueError("A 'swat.CAS' session can only be reused "
-                                 "when it's connected via the REST APIs.")
+                raise ValueError(
+                    "A 'swat.CAS' session can only be reused "
+                    "when it's connected via the REST APIs."
+                )
         else:
             url = urlsplit(hostname)
 
@@ -334,20 +350,22 @@ class Session(requests.Session):
 
             domain = url.hostname or str(hostname)
 
-        self._settings = {'protocol': protocol or 'https',
-                          'domain': domain,
-                          'port': port,
-                          'username': username,
-                          'password': password
-                          }
+        self._settings = {
+            'protocol': protocol or 'https',
+            'domain': domain,
+            'port': port,
+            'username': username,
+            'password': password,
+        }
 
         if self._settings['password'] is None:
             # Try to get credentials from .authinfo or .netrc files.
             # If no file path was specified, the default locations will
             # be checked.
             if 'swat' in sys.modules:
-                auth = swat.utils.authinfo.query_authinfo(domain, user=username,
-                                                          path=authinfo)
+                auth = swat.utils.authinfo.query_authinfo(
+                    domain, user=username, path=authinfo
+                )
                 self._settings['username'] = auth.get('user')
                 self._settings['password'] = auth.get('password')
 
@@ -359,9 +377,11 @@ class Session(requests.Session):
                     parser = netrc.netrc(authinfo)
                     values = parser.authenticators(domain)
                     if values:
-                        self._settings['username'], \
-                        _, \
-                        self._settings['password'] = values
+                        (
+                            self._settings['username'],
+                            _,
+                            self._settings['password'],
+                        ) = values
                 except (OSError, IOError):
                     pass  # netrc throws if $HOME is not set
 
@@ -417,6 +437,74 @@ class Session(requests.Session):
         """
         return self.add_logger(logging.StreamHandler(), level)
 
+    @versionadded(version='1.6')
+    def as_swat(self, server=None, **kwargs):
+        """Create a SWAT connection to a CAS server.
+
+        Uses the authentication information from the session to establish a CAS connection using SWAT.
+
+        Parameters
+        ----------
+        server : str, optional
+            The logical name of the CAS server, not the hostname.  Defaults to "cas-shared-default".
+        kwargs : any
+            Additional arguments to pass to the `swat.CAS` constructor.  Can be used to override this method's
+            default behavior or customize the CAS session.
+
+        Returns
+        -------
+        swat.CAS
+            An active SWAT connection
+
+        Raises
+        ------
+        RuntimeError
+            If `swat` package is not available.
+
+        Examples
+        --------
+        >>> sess = Session('example.sas.com')
+        >>> with sess.as_swat() as conn:
+        ...    conn.listnodes()
+        CASResults([('nodelist', Node List
+                      name        role connected   IP Address
+        0  example.sas.com  controller       Yes  127.0.0.1)])
+
+        """
+        server = server or 'cas-shared-default'
+
+        if swat is None:
+            raise RuntimeError(
+                "The 'swat' package must be installed to create a SWAT connection."
+            )
+
+        # Construct the CAS server's URL
+        url = '{}://{}/{}-http/'.format(
+            self._settings['protocol'], self.hostname, server
+        )
+
+        # Use this sessions info to connect to CAS unless user has explicitly give a value (even if None)
+        kwargs.setdefault('hostname', url)
+        kwargs.setdefault('username', self.username)
+        kwargs.setdefault('password', self._settings['password'])
+
+        orig_sslreqcert = os.environ.get('SSLREQCERT')
+
+        # If SSL connections to microservices are not being verified, don't attempt
+        # to verify connections to CAS - most likely certs are not in place.
+        if not self.verify:
+            os.environ['SSLREQCERT'] = 'no'
+
+        try:
+            cas = swat.CAS(**kwargs)
+            cas.setsessopt(messagelevel='warning')
+        finally:
+            # Reset environment variable to whatever it's original value was
+            if orig_sslreqcert:
+                os.environ['SSLREQCERT'] = orig_sslreqcert
+
+        return cas
+
     @property
     def username(self):
         return self._settings.get('username')
@@ -436,11 +524,13 @@ class Session(requests.Session):
                     verb=r.method,
                     url=r.url,
                     headers='\n'.join(
-                        '{}: {}'.format(k, v) for k, v in r.headers.items()),
-                    body=_pformat(r.body)))
+                        '{}: {}'.format(k, v) for k, v in r.headers.items()
+                    ),
+                    body=_pformat(r.body),
+                )
+            )
         else:
-            self.message_log.info('HTTP/1.1 %s %s', request.method,
-                                  request.url)
+            self.message_log.info('HTTP/1.1 %s %s', request.method, request.url)
 
         response = super(Session, self).send(request, **kwargs)
 
@@ -454,35 +544,66 @@ class Session(requests.Session):
                     status=r.status_code,
                     url=r.url,
                     headers='\n'.join(
-                        '{}: {}'.format(k, v) for k, v in r.headers.items()),
-                    body=_pformat(r.text)))
+                        '{}: {}'.format(k, v) for k, v in r.headers.items()
+                    ),
+                    body=_pformat(r.text),
+                )
+            )
         else:
-            self.message_log.info('HTTP/1.1 %s %s', response.status_code,
-                                  response.url)
+            self.message_log.info('HTTP/1.1 %s %s', response.status_code, response.url)
 
         return response
 
-    def request(self, method, url,
-                params=None, data=None, headers=None, cookies=None, files=None,
-                auth=None, timeout=None, allow_redirects=True, proxies=None,
-                hooks=None, stream=None, verify=None, cert=None, json=None):
+    def request(
+        self,
+        method,
+        url,
+        params=None,
+        data=None,
+        headers=None,
+        cookies=None,
+        files=None,
+        auth=None,
+        timeout=None,
+        allow_redirects=True,
+        proxies=None,
+        hooks=None,
+        stream=None,
+        verify=None,
+        cert=None,
+        json=None,
+    ):
 
         url = self._build_url(url)
         verify = verify or self.verify
 
         try:
-            return super(Session, self).request(method, url, params, data,
-                                                headers, cookies, files, auth,
-                                                timeout, allow_redirects,
-                                                proxies, hooks, stream, verify,
-                                                cert, json)
+            return super(Session, self).request(
+                method,
+                url,
+                params,
+                data,
+                headers,
+                cookies,
+                files,
+                auth,
+                timeout,
+                allow_redirects,
+                proxies,
+                hooks,
+                stream,
+                verify,
+                cert,
+                json,
+            )
         except requests.exceptions.SSLError as e:
             if 'REQUESTS_CA_BUNDLE' not in os.environ:
                 raise RuntimeError(
                     "SSL handshake failed.  The 'REQUESTS_CA_BUNDLE' "
                     "environment variable should contain the path to the CA "
                     "certificate.  Alternatively, set verify_ssl=False to "
-                    "disable certificate verification.")
+                    "disable certificate verification."
+                )
             raise e
 
     def get(self, url, **kwargs):
@@ -503,8 +624,10 @@ class Session(requests.Session):
     def _get_token_with_kerberos(self):
         """Authenticate with a Kerberos ticket."""
         if kerberos is None:
-            raise RuntimeError("Kerberos package not found.  Run 'pip "
-                               "install sasctl[kerberos]' to install.")
+            raise RuntimeError(
+                "Kerberos package not found.  Run 'pip "
+                "install sasctl[kerberos]' to install."
+            )
 
         user = self._settings.get('username')
         # realm = user.rsplit('@', maxsplit=1)[-1] if '@' in user else None
@@ -512,35 +635,36 @@ class Session(requests.Session):
         flags = kerberos.GSS_C_MUTUAL_FLAG | kerberos.GSS_C_SEQUENCE_FLAG
         service = 'HTTP@%s' % self._settings['domain']
 
-        logger.info('Attempting Kerberos authentication to %s as %s',
-                    service, user)
+        logger.info('Attempting Kerberos authentication to %s as %s', service, user)
 
         url = self._build_url(
-            '/SASLogon/oauth/authorize?client_id=%s&response_type=token'
-            % client_id)
+            '/SASLogon/oauth/authorize?client_id=%s&response_type=token' % client_id
+        )
 
         # Get Kerberos challenge
         r = self.get(url, allow_redirects=False, verify=self.verify)
 
         if r.status_code != 401:
-            raise ValueError('Kerberos challenge response not received.  '
-                             'Expected HTTP 401 but received %s' %
-                             r.status_code)
+            raise ValueError(
+                'Kerberos challenge response not received.  '
+                'Expected HTTP 401 but received %s' % r.status_code
+            )
 
         if 'www-authenticate' not in r.headers:
-            raise ValueError("Kerberos challenge response not received.  "
-                             "'WWW-Authenticate' header not received.")
+            raise ValueError(
+                "Kerberos challenge response not received.  "
+                "'WWW-Authenticate' header not received."
+            )
 
         if 'Negotiate' not in r.headers['www-authenticate']:
-            raise ValueError("Kerberos challenge response not received.  "
-                             "'WWW-Authenticate' header contained '%s', "
-                             "expected 'Negotiate'."
-                             % r.headers['www-authenticate'])
+            raise ValueError(
+                "Kerberos challenge response not received.  "
+                "'WWW-Authenticate' header contained '%s', "
+                "expected 'Negotiate'." % r.headers['www-authenticate']
+            )
 
         # Initialize a request to KDC for a ticket to access the service.
-        _, context = kerberos.authGSSClientInit(service,
-                                                principal=user,
-                                                gssflags=flags)
+        _, context = kerberos.authGSSClientInit(service, principal=user, gssflags=flags)
 
         # Send the request.
         # NOTE: empty-string parameter required for initial call.
@@ -558,20 +682,25 @@ class Session(requests.Session):
             self._settings['username'] = username.rsplit('@', maxsplit=1)[0]
 
         # Response to Kerberos challenge with ticket
-        r = self.get(url,
-                     headers={'Authorization': auth_header},
-                     allow_redirects=False,
-                     verify=self.verify)
+        r = self.get(
+            url,
+            headers={'Authorization': auth_header},
+            allow_redirects=False,
+            verify=self.verify,
+        )
 
         if 'Location' not in r.headers:
-            raise ValueError("Invalid authentication response."
-                             "'Location' header not received.")
+            raise ValueError(
+                "Invalid authentication response." "'Location' header not received."
+            )
 
         match = re.search('(?<=access_token=)[^&]*', r.headers['Location'])
 
         if match is None:
-            raise ValueError("Invalid authentication response.  'Location' "
-                             "header does not contain an access token.")
+            raise ValueError(
+                "Invalid authentication response.  'Location' "
+                "header does not contain an access token."
+            )
 
         return match.group(0)
 
@@ -581,15 +710,15 @@ class Session(requests.Session):
         password = self._settings['password']
         url = self._build_url('/SASLogon/oauth/token')
 
-        data = 'grant_type=password&username={}&password={}'.format(username,
-                                                                    password)
-        headers = {'Accept': 'application/json',
-                   'Content-Type': 'application/x-www-form-urlencoded'}
+        data = 'grant_type=password&username={}&password={}'.format(username, password)
+        headers = {
+            'Accept': 'application/json',
+            'Content-Type': 'application/x-www-form-urlencoded',
+        }
 
-        r = super(Session, self).post(url,
-                                      data=data,
-                                      headers=headers,
-                                      auth=('sas.ec', ''))
+        r = super(Session, self).post(
+            url, data=data, headers=headers, auth=('sas.ec', '')
+        )
 
         if r.status_code == 401:
             raise exceptions.AuthenticationError(username)
@@ -629,13 +758,15 @@ class Session(requests.Session):
         if self._settings['port'] is not None and ':' not in domain:
             domain = '{}:{}'.format(domain, self._settings['port'])
 
-        return urlunsplit([
-            components.scheme or self._settings['protocol'],
-            domain,
-            components.path,
-            components.query,
-            components.fragment
-        ])
+        return urlunsplit(
+            [
+                components.scheme or self._settings['protocol'],
+                domain,
+                components.path,
+                components.query,
+                components.fragment,
+            ]
+        )
 
     def __enter__(self):
         super(Session, self).__enter__()
@@ -653,13 +784,16 @@ class Session(requests.Session):
         super(Session, self).__exit__()
 
     def __str__(self):
-        return ("{class_}(hostname='{hostname}', username='{username}', "
-                "protocol='{protocol}', verify_ssl={verify})".format(
-            class_=type(self).__name__,
-            hostname=self.hostname,
-            username=self.username,
-            protocol=self._settings.get('protocol'),
-            verify=self.verify))
+        return (
+            "{class_}(hostname='{hostname}', username='{username}', "
+            "protocol='{protocol}', verify_ssl={verify})".format(
+                class_=type(self).__name__,
+                hostname=self.hostname,
+                username=self.username,
+                protocol=self._settings.get('protocol'),
+                verify=self.verify,
+            )
+        )
 
 
 class PageIterator:
@@ -686,6 +820,7 @@ class PageIterator:
         Items contained in the current page
 
     """
+
     def __init__(self, obj, session=None, threads=4):
         self._num_threads = threads
 
@@ -709,9 +844,13 @@ class PageIterator:
 
             # Construct a new link with format params
             # Result is "/spam/spam?start={start}&limit={limit}"
-            link = link[:start.start()] + '{start}' \
-                   + link[start.end():limit.start()] \
-                   + '{limit}' + link[limit.end():]
+            link = (
+                link[: start.start()]
+                + '{start}'
+                + link[start.end() : limit.start()]
+                + '{limit}'
+                + link[limit.end() :]
+            )
 
             self._start = int(start.group())
             self._limit = int(limit.group())
@@ -725,11 +864,10 @@ class PageIterator:
         self._obj = obj
 
     def __next__(self):
-        return self.next()
-
-    def next(self):
         if self._pool is None:
-            self._pool = concurrent.futures.ThreadPoolExecutor(max_workers=self._num_threads)
+            self._pool = concurrent.futures.ThreadPoolExecutor(
+                max_workers=self._num_threads
+            )
 
         # Send request for new pages if we don't have enough cached
         num_req_needed = self._num_threads - len(self._requested)
@@ -759,6 +897,8 @@ class PageIterator:
         # All Iterators are also Iterables
         return self
 
+    next = __next__  # Python 2 compatible
+
     def _request_async(self, start):
         """Used by worker threads to retrieve next batch of items."""
 
@@ -775,6 +915,9 @@ class PageIterator:
 class PagedItemIterator:
     """Iterates through a collection that must be "paged" from the server.
 
+    Uses `PageIterator` to transparently download pages of items from the server
+    as needed.
+
     Parameters
     ----------
     obj : RestObj
@@ -790,78 +933,138 @@ class PagedItemIterator:
     ------
     RestObj
 
+    Notes
+    -----
+    Value returned by len() is an approximate count of the items available.  The actual
+    number of items returned may be greater than or less than this number.
+
+    See Also
+    --------
+    PageIterator
+
     """
+
     def __init__(self, obj, session=None, threads=4):
         # Iterates over whole pages of items
         self._pager = PageIterator(obj, session, threads)
 
+        # Store items from latest page that haven't been returned yet.
+        self._cache = []
+
         # Total number of items to iterate over
         if 'count' in obj:
+            # NOTE: "count" may be an (over) estimate of the number of items available
+            #       since some may be inaccessible due to user permissions & won't actually be returned.
             self._count = int(obj.count)
         else:
             self._count = len(obj['items'])
 
-        self._cache = []
-
     def __len__(self):
         return self._count
 
     def __next__(self):
-        return self.next()
-
-    def next(self):
         # Get next page of items if we're currently out
         if not self._cache:
             self._cache = next(self._pager)
 
+            if len(self._cache) < self._pager._limit:
+                # number of items returned in page was less than expected
+                # might be last page, or items might have been filtered out by server.
+                pass
+
         # Return the next item
         if self._cache:
+            self._count -= 1
             return self._cache.pop(0)
 
-        raise StopIteration
+        raise StopIteration()
+        # Out of items and out of pages
+        # self._count = 0
+        # raise StopIteration
 
     def __iter__(self):
         return self
 
+    next = __next__  # Python 2 compatible
+
+
+class PagedListIterator:
+    """Iterates over an instance of PagedList
+
+    Parameters
+    ----------
+    l : list-like
+
+    """
 
-class PagedListIterator():
     def __init__(self, l):
         self.__list = l
         self.__index = 0
 
     def __next__(self):
-        return self.next()
-
-    def next(self):
         if self.__index >= len(self.__list):
             raise StopIteration
 
-        item = self.__list[self.__index]
-        self.__index += 1
-        return item
+        try:
+            item = self.__list[self.__index]
+            self.__index += 1
+            return item
+        except IndexError:
+            # Because PagedList length is approximate, iterating can result in
+            # indexing outside the array.  Just stop the iteration if that occurs.
+            raise StopIteration
 
     def __iter__(self):
         return self
 
+    next = __next__  # Python 2 compatibility
+
 
 class PagedList(list):
-    def __init__(self, obj, **kwargs):
+    """List that dynamically loads items from the server.
+
+    Parameters
+    ----------
+    obj : RestObj
+        An instance of `RestObj` containing any initial items and a link to
+        retrieve additional items.
+    session : Session, optional
+        The `Session` instance to use for requesting additional items.  Defaults
+        to current_session()
+    threads : int, optional
+        Number of threads allocated to loading additional items.
+
+    Notes
+    -----
+    Value returned by len() is an approximate count of the items available.  The actual
+    length is not known until all items have been pulled from the server.
+
+    See Also
+    --------
+    PagedItemIterator
+
+    """
+
+    def __init__(self, obj, session=None, threads=4):
         super(PagedList, self).__init__()
-        self._pager = PagedItemIterator(obj, **kwargs)
+        self._pager = PagedItemIterator(obj, session=session, threads=threads)
 
-        # Force caching of first page
-        self.append(next(self._pager))
-        items = self._pager._cache
-        self.extend(items)
+        # Add the first page of items to the list
+        for _ in range(len(self._pager._cache)):
+            self.append(next(self._pager))
 
-        # clear the list (py27 compatible)
-        del self._pager._cache[:]
+        # Assume that server has more items available
+        self._has_more = True
 
     def __len__(self):
-        return len(self._pager)
+        if self._has_more:
+            # Estimate the total length as items downloaded + items still on server
+            return super(PagedList, self).__len__() + len(self._pager)
+        else:
+            # We've pulled everything from the server, so we have an exact length now.
+            return super(PagedList, self).__len__()
 
     def __iter__(self):
-        # return super(PagedList, self).__iter__()
         return PagedListIterator(self)
 
     def __getslice__(self, i, j):
@@ -877,15 +1080,22 @@ class PagedList(list):
         else:
             idx = int(item)
 
+            # Support negative indexing.  Need to load items up to len() - idx.
+            if idx < 0:
+                idx = len(self) + idx
+
         try:
+            # Iterate through server-side pages until we've loaded
+            # the item at the requested index.
             while super(PagedList, self).__len__() <= idx:
                 n = next(self._pager)
                 self.append(n)
-        except StopIteration:
-            # May cause if slice or index extends beyond array
-            # Ignore and let List handle IndexErrors if necessary.
-            pass
 
+        except StopIteration:
+            # We've hit the end of the paging so the server has no more items to retrieve.
+            self._has_more = False
+
+        # Get the item from the list
         return super(PagedList, self).__getitem__(item)
 
     def __str__(self):
@@ -1053,9 +1263,11 @@ def request(verb, path, session=None, raw=False, format='auto', **kwargs):
         raise TypeError('No `Session` instance found.')
 
     if raw:
-        warnings.warn("The 'raw' parameter is deprecated and will be removed in"
-                      " a future version.  Use format='response' instead.",
-                      DeprecationWarning)
+        warnings.warn(
+            "The 'raw' parameter is deprecated and will be removed in"
+            " a future version.  Use format='response' instead.",
+            DeprecationWarning,
+        )
         format = 'response'
 
     format = 'auto' if format is None else str(format).lower()
@@ -1065,8 +1277,9 @@ def request(verb, path, session=None, raw=False, format='auto', **kwargs):
     response = session.request(verb, path, **kwargs)
 
     if 400 <= response.status_code <= 599:
-        raise HTTPError(response.url, response.status_code, response.text,
-                        response.headers, None)
+        raise HTTPError(
+            response.url, response.status_code, response.text, response.headers, None
+        )
 
     # Return the raw response if requested
     if format == 'response':
@@ -1189,8 +1402,7 @@ def _unwrap(json):
     return RestObj(json)
 
 
-def _build_crud_funcs(path, single_term=None, plural_term=None,
-                      service_name=None):
+def _build_crud_funcs(path, single_term=None, plural_term=None, service_name=None):
     """Utility method for defining simple functions to perform CRUD operations on a REST endpoint.
 
     Parameters
@@ -1310,11 +1522,12 @@ def _build_crud_funcs(path, single_term=None, plural_term=None,
 
         id_ = getattr(item, 'id', None)
         if id_ is None:
-            raise ValueError(
-                'Could not find property `id` for update of %s.' % item)
+            raise ValueError('Could not find property `id` for update of %s.' % item)
 
-        headers = {'If-Match': item._headers.get('etag'),
-                   'Content-Type': item._headers.get('content-type')}
+        headers = {
+            'If-Match': item._headers.get('etag'),
+            'Content-Type': item._headers.get('content-type'),
+        }
 
         return put(path + '/%s' % id_, json=item, headers=headers)
 
@@ -1371,4 +1584,29 @@ def _build_is_available_func(service_root):
         """
         response = current_session().head(service_root + '/')
         return response.status_code == 200
+
     return is_available
+
+
+@versionadded(version='1.5.6')
+def platform_version():
+    """Get the version of the SAS Viya platform to which sasctl is connected.
+
+    Returns
+    -------
+    string : {'3.5', '4.0'}
+
+        SAS Viya version number
+
+    """
+    from .services import model_repository as mr
+
+    response = mr.info()
+    buildVersion = response.get('build')['buildVersion']
+    try:
+        if buildVersion[0:4] == '3.7.':
+            return '3.5'
+        elif float(buildVersion[0:4]) >= 3.10:
+            return '4.0'
+    except ValueError:
+        pass  # Version could not be found.  Return None instead.
