@@ -2,7 +2,7 @@
 # SPDX-License-Identifier: Apache-2.0
 
 from .pickleModel import PickleModel
-from .uploadData import ModelImport
 from .writeJSONFiles import JSONFiles
 from .zipModel import ZipModel
 from .writeScoreCode import ScoreCode
+from .importModel import ImportModel
