@@ -4,12 +4,14 @@
 # %%
 from pathlib import Path
 import zipfile
+import io
 
 # %%
 
 class ZipModel():
     
-    def zipFiles(self, fileDir, modelPrefix):
+    @staticmethod
+    def zipFiles(fileDir, modelPrefix):
         '''
         Combines all JSON files with the model pickle file and associated score code file
         into a single archive ZIP file.
@@ -33,7 +35,11 @@ class ZipModel():
         fileNames.extend(sorted(Path(fileDir).glob('*.json')))
         fileNames.extend(sorted(Path(fileDir).glob('*Score.py')))
         fileNames.extend(sorted(Path(fileDir).glob('*.pickle')))
+        fileNames.extend(sorted(Path(fileDir).glob('*.mojo')))
         
         with zipfile.ZipFile(Path(fileDir) / (modelPrefix + '.zip'), mode='w') as zFile:
             for name in fileNames:
-                zFile.write(name, arcname=name)
\ No newline at end of file
+                zFile.write(name, arcname=name)
+                
+        with open(Path(fileDir) / (modelPrefix + '.zip'), 'rb') as zipFile:
+            return io.BytesIO(zipFile.read())
